#!/usr/bin/perl

use strict;
use JSON::XS;
use Spreadsheet::WriteExcel;

die "$0 expects two filenames, aborting !" unless $#ARGV;

my $plan = shift;
my $dest = shift;

open( F, $plan ) || die "Can't open JSON test plan, aborting !";
my $f = join( "", <F> );
close F;
my $json_obj = JSON::XS->new->allow_nonref(1);
my $json     = $json_obj->decode($f);

#require Data::Dumper;
#print "JSON content is: ", Data::Dumper::Dumper( $json ), "\n";

my $workbook = Spreadsheet::WriteExcel->new($dest);
$workbook->compatibility_mode();
my $worksheet4 = $workbook->add_worksheet('Test Descriptions');
$worksheet4->freeze_panes( 6, 0 );
$_->outline_settings( 1, 0, 0, 1 ) for $workbook->sheets();
$_->hide_gridlines(2) for $workbook->sheets();
$workbook->set_custom_color( 22, 230, 230, 230 );    # == Gray 10% (OOo)
$workbook->set_custom_color( 53, 220, 76,  0 );      # == Ubuntu Orange
my $i = 1;

my $format01 = $workbook->add_format(                # Titles
    align => 'left',
    size  => 12,
    bold  => 1,
);
my $format02 = $workbook->add_format(                # Headlines
    align => 'center',
    size  => 10,
    bold  => 1,
);
my $format03 = $workbook->add_format(
    align     => 'left',
    valign    => 'vcenter',
    text_wrap => 1,
    size      => 8,
);
my $format04 = $workbook->add_format(
    align     => 'left',
    valign    => 'vcenter',
    text_wrap => 1,
    size      => 8,
    bg_color  => 22,
);
my $format05 = $workbook->add_format(
    align => 'left',
    size  => 24,
    color => 53,
);
my $format10 = $workbook->add_format(    # Comments with indent lvl 1
    align     => 'left',
    valign    => 'vcenter',
    text_wrap => 1,
    size      => 8,
    indent    => 1,
);
my $format11 = $workbook->add_format(    # Comments with indent lvl 1
    align     => 'left',
    valign    => 'vcenter',
    text_wrap => 1,
    size      => 8,
    indent    => 1,
    bg_color  => 22,
);
my $format12 = $workbook->add_format(    # Test categories
    align => 'left',
    size  => 10,
    bold  => 1,
);
$_->set_font('Verdana') for @{ $workbook->{_formats} };

# Create the test descriptions sheet
$i = 1;
$worksheet4->write( $i, 1, 'System Test Plan', $format05 );
$worksheet4->set_row( $i, 30 );
$i += 2;
$worksheet4->set_row( $i, 15 );
$worksheet4->write( $i++, 1, 'Tests Descriptions', $format01 );
$i++;

my $level     = 1;
my $max_level = 1;

sub find_max_level {    # Find the maximum depth of the test tree
    my $tree = shift;
    foreach my $job ( @{$tree} ) {
        if ( scalar @{ $job->{'children'} } ) {
            $level++;
            find_max_level( $job->{'children'} );
            $level--;
        }
        else {
            $max_level = $level if $level > $max_level;
        }
    }
}

sub write_test {
    my $tree = shift;
    foreach my $job ( @{$tree} ) {
        unless ( scalar @{ $job->{'children'} } ) {
            my $name = $job->{'description'}->{'name'};
            my $desc = $job->{'description'}->{'description'};
            s/\n//g for $name;
            $name =~ s/.*\///;
            my @lines = $desc =~ /\n/gm;
            my $lines = scalar(@lines);
            $worksheet4->set_row( $i, 12 + 11 * $lines, undef, 0, $level )
              if $level;
            $worksheet4->write( $i, $max_level + 1,
                $name, $i % 2 ? $format04 : $format03 );

            $worksheet4->write( $i, $max_level + 2,
                $desc, $i % 2 ? $format11 : $format10 );
            $i++;
        }
        else {
            $worksheet4->set_row( $i, undef, undef, 0, $level, 0 );
            if ( ref $job->{'description'} eq 'HASH' ) {
                $worksheet4->write( $i++, $level + 1,
                    $job->{'description'}->{'name'}, $format12 );
            }
            else {
                my $title = $job->{'description'};
                $title =~ s/\.txt(\.in)?$//;
                $worksheet4->write( $i++, $level + 1, $title, $format12 );
            }
            $level++;
            write_test( $job->{'children'} );
            $level--;
        }
    }
}

find_max_level($json);
$level = 0;

$worksheet4->set_column( 0, 0, 5 );
$worksheet4->set_column( $_, $_, 2 ) for ( 1 .. $max_level );
$worksheet4->set_column( $max_level + 1, $max_level + 1, 55 );
$worksheet4->set_column( $max_level + 2, $max_level + 2, 90 );

$worksheet4->write( $i++, $max_level + 1, [ 'Name', 'Description' ],
    $format02 );

write_test($json);
$worksheet4->autofilter( 5, $max_level + 1, $i, $max_level + 2 );
