#!/usr/bin/python
import os, re
from distutils.core import setup
from DistUtilsExtra.command.build_extra import build_extra
from DistUtilsExtra.command.build_icons import build_icons


def changelog_version(changelog="debian/changelog"):
    version="dev"
    if os.path.exists(changelog):
        head=open(changelog).readline()
        match=re.match(".*\((.*)\).*", head)
        if match:
            version=match.group(1)
    return version


class checkbox_editor_build_icons(build_icons, object):
    def initialize_options(self):
        super(checkbox_editor_build_icons, self).initialize_options()
        self.icon_dir = "icons"


def get_template_files():
    top = 'templates'
    template_files = []
    for dirpath, dirnames, filenames in os.walk(top):
        filenames = [os.path.join(dirpath, filename)
                     for filename in filenames
                     if not filename.endswith('~')]
        if filenames:
            share_path = os.path.join('share/checkbox-editor', dirpath)
            template_files.append((share_path, filenames))
    return template_files

setup(
    name = "checkbox-editor",
    version = changelog_version(),
    author = "Javier Collado",
    author_email = "javier.collado@canonical.com",
    url = "https://code.launchpad.net/~oem-community-qa/checkbox-editor/trunk",
    license = "GPL",
    description = "Editor for the checkbox test integration framework",
    long_description = """
Checkbox test suites and cases are written by editing different files
in plain text format. This application provides a front end to those
files to make it easier to edit them easily in a user-friendly user
interface.""",
    packages = ['checkbox_editor'],
    package_data={'checkbox_editor': ['glade/*',
                                      'script/*.pl',
                                      'help/*.xml',
                                      'help/figures/*']},
    scripts = ['bin/checkbox-editor'],
    data_files = [('share/applications', ['gtk/checkbox-editor.desktop']),
                  ('share/apport/package-hooks', ['apport/source_checkbox-editor.py']),
                  ('/etc/apport/crashdb.conf.d', ['apport/checkbox-editor-crashdb.conf']),
                  ] + get_template_files(),
    cmdclass = {
        "build": build_extra,
        "build_icons": checkbox_editor_build_icons,
        }
)
